#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mkl_cblas.h"
#include <offload.h>
//include <mkl_cblas.h>

#include <sys/time.h>
double mysecond();

#define  LOOPS 3
 
 // MKL DGEMM call is offload to the MIC.
 // Use this as a template for offloading your own functions.
 // C is row-major order.

void matgen(char cmat, int m, int n, int lda, double *a);

void mxm(int m, int n, int k, int ldmax,
         double *a,
         double *b,
         double *c);

__attribute__((target(mic)))
void cblas_dgemm(const  CBLAS_ORDER Order, const  CBLAS_TRANSPOSE TransA,
                 const  CBLAS_TRANSPOSE TransB, const MKL_INT M, const MKL_INT N,
                 const MKL_INT K, const double alpha, const double *A,
                 const MKL_INT lda, const double *B, const MKL_INT ldb,
                 const double beta, double *C, const MKL_INT ldc);


int main(void){
      int isizes[] = { 1, 3, 7 };
//                                    hold clock times
      double c0,c1,cd;

//                                    dgemm parameter variables
      char    transa, transb;

      int     m, n, k, morder;
      int     i, j, loop;

      double  alpha, beta;
      double *a, *b, *c  ;

      transa='n';
      transb='n';
      alpha = 1.0e0;
      beta  = 1.0e0;

      printf("Square Matrix x Matrix, AxB=C\n");
//                                               Print out mxm parameters
      printf("Input:\n"); 
      printf("   max mat order =%8i\n",         100*(1<<isizes[LOOPS-1]) );
      printf("   alpha, beta   =%4.0f%4.0f\n",  alpha,   beta);
      printf("   transa transb =   %c   %c\n", transa, transb);

//                                               loop over 3 trials
//                                               throw the first one away
   for(loop=0; loop<LOOPS; loop++){

      morder = 100*(1<<isizes[loop]);
//                                               all matrices are square and same order
      m=morder;
      n=morder;
      k=morder;
//                                               allocate memory for each trial
      a = (double *) malloc( (size_t) n*k*sizeof(double) );
      b = (double *) malloc( (size_t) k*n*sizeof(double) );
      c = (double *) malloc( (size_t) n*m*sizeof(double) );

//                                               initialize all matrices
      matgen('a',n,k, k,a);
      matgen('b',k,m, m,b);
      matgen('c',n,m, m,c);

      c0 = mysecond();

//                                               Offload to a MKL dgemm mxm routine
//                                               and record times
//        C=alpha*AxB+beta*C     Optimize data transfers:
//                               by using "in" clause for a and b matrices
      #pragma offload target(mic:0) inout(a,b,c: length(morder*morder))
      cblas_dgemm(CblasRowMajor, CblasNoTrans,  
                  CblasNoTrans,  m, n, k,
                  alpha, a, k,
                         b, m, 
                   beta, c, m);
      c1 = mysecond();
//                                               don't print cost of first offload
      if(loop!=0) printf(" %6d (order) %13.6f (sec)   mxm\n",morder,c1-c0);

   }

}
